package tbox;

import java.util.*;

import assertions.*;

public class TBox {
	private Alfabeth alfabeth;
	private Collection<InclusionAssertion> inclusionAssertions;
	private Collection<FunctionalityAssertion> functionalityAssertions;
	private Collection<IdentificationAssertion> identificationAssertions;
	private Collection<ISA> lista_isa;
	
	
	public TBox() {
		super();
		this.inclusionAssertions = new Vector<InclusionAssertion>();
		this.functionalityAssertions = new Vector<FunctionalityAssertion>();
		this.identificationAssertions = new Vector<IdentificationAssertion>();
		this.alfabeth = new Alfabeth();
		this.lista_isa = new Vector<ISA>();
	}


	public TBox(Collection<InclusionAssertion> inclusionAssertions,
			Collection<FunctionalityAssertion> functionalityAssertions,
			Collection<IdentificationAssertion> identificationAssertions,
			Alfabeth alfabeth, Collection<ISA> lista_isa) {
		super();
		this.inclusionAssertions = inclusionAssertions;
		this.functionalityAssertions = functionalityAssertions;
		this.identificationAssertions = identificationAssertions;
		this.alfabeth = alfabeth;
		this.lista_isa = lista_isa;
	}


	public Collection<InclusionAssertion> getInclusionAssertions() {
		return inclusionAssertions;
	}


	public void setInclusionAssertions(
			Collection<InclusionAssertion> inclusionAssertions) {
		this.inclusionAssertions = inclusionAssertions;
	}


	public Collection<FunctionalityAssertion> getFunctionalityAssertions() {
		return functionalityAssertions;
	}


	public void setFunctionalityAssertions(
			Collection<FunctionalityAssertion> functionalityAssertions) {
		this.functionalityAssertions = functionalityAssertions;
	}


	public Collection<IdentificationAssertion> getIdentificationAssertions() {
		return identificationAssertions;
	}


	public void setIdentificationAssertions(
			Collection<IdentificationAssertion> identificationAssertions) {
		this.identificationAssertions = identificationAssertions;
	}

	
	
	public Collection<ISA> getLista_isa() {
		return lista_isa;
	}


	public void setLista_isa(Collection<ISA> lista_isa) {
		this.lista_isa = lista_isa;
	}


	public Alfabeth getAlfabeth() {
		return alfabeth;
	}


	public void setAlfabeth(Alfabeth alfabeth) {
		this.alfabeth = alfabeth;
	}
	
	public void addAssertion(InclusionAssertion ass) {
		this.inclusionAssertions.add(ass);
	}
	
	public void addAssertion(FunctionalityAssertion ass) {
		this.functionalityAssertions.add(ass);
	}
	
	public void addAssertion(IdentificationAssertion ass) {
		this.identificationAssertions.add(ass);
	}
	
	public void addISA(ISA isa) {
		this.lista_isa.add(isa);
	}
	
	public String toString() {
		String desc = "TBox:\n";

		if (alfabeth!=null) desc = desc.concat(this.alfabeth.toString());
		
		Iterator i = this.inclusionAssertions.iterator();
		if (i.hasNext()) desc = desc.concat("\n**Inclusion Assertions:");
		while(i.hasNext()) {
			desc = desc.concat("\n  " + ((InclusionAssertion) i.next()).toString());
		}
		i = this.functionalityAssertions.iterator();
		if (i.hasNext()) desc = desc.concat("\n**Functionality Assertions:");
		while(i.hasNext()) {
			desc = desc.concat("\n  " + ((FunctionalityAssertion) i.next()).toString());
		}

		i = this.identificationAssertions.iterator();
		if (i.hasNext()) desc = desc.concat("\n**Identification Constraint Assertions:");
		while(i.hasNext()) {
			desc = desc.concat("\n  " + ((IdentificationAssertion) i.next()).toString());
		}
		return desc;
	}
	
}
