package translation;

import java.util.*;

import tbox.*;
import domain.*;
import assertions.*;

public class FOLTranslator {
	public static String translate(TBox t) {
		String folDesc = "%% Rappresentazione FOL della TBox:\n";

		if (t.getAlfabeth()!=null) folDesc = folDesc.concat(t.getAlfabeth().toFOLPredicates()+"\n");
		
		Iterator i = t.getInclusionAssertions().iterator();
		if (i.hasNext()) folDesc = folDesc.concat("\n%%**Inclusion Assertions:\n");
		while(i.hasNext()) {
			InclusionAssertion ia = (InclusionAssertion)i.next();
			String folIa = translateInclusionAssertion(t, ia);
			folDesc = folDesc.concat(folIa+"\n");
		}
		i = t.getFunctionalityAssertions().iterator();
		if (i.hasNext()) folDesc = folDesc.concat("\n%%**Functionality Assertions:\n");
		while(i.hasNext()) {
			FunctionalityAssertion fa = (FunctionalityAssertion)i.next();
			String folFa = translateFunctionalityAssertion(fa);
			folDesc = folDesc.concat(folFa+"\n");
		}

		i = t.getIdentificationAssertions().iterator();
		if (i.hasNext()) folDesc = folDesc.concat("\n%%**Identification Constraint Assertions:\n");
		while(i.hasNext()) {
			IdentificationAssertion ida = (IdentificationAssertion)i.next();
			String folIda = translateIdentificationAssertion(ida);
			folDesc = folDesc.concat(folIda+"\n");
		}
		folDesc = folDesc.concat(valueDisjointness(t));
		folDesc = folDesc.concat(conceptDisjointness(t));
		
		return folDesc;
	}
	
	private static String valueDisjointness(TBox t) {
		String temp = "";
		Collection<AtomicValueDomain> valuesets = t.getAlfabeth().getValueSets();
		Object[] arr = valuesets.toArray();
		for (int i=0; i< arr.length; i++)
			for(int j=i+1; j<arr.length; j++) {
				AtomicValueDomain sx = (AtomicValueDomain)arr[i];
				AtomicValueDomain dx = (AtomicValueDomain)arr[j];
				temp = temp.concat("all X ("+sx.getName()+"(X) -> -"+dx.getName()+"(X)).\n");
			}
				
		return temp;
	}
	
	private static String conceptDisjointness(TBox t) {
		String temp = "";
		Collection<AtomicConcept> concepts = t.getAlfabeth().getConcepts();
		Object[] arr = concepts.toArray();
		for (int i=0; i< arr.length; i++)
			for(int j=i+1; j<arr.length; j++) {
				AtomicConcept sx = (AtomicConcept)arr[i];
				AtomicConcept dx = (AtomicConcept)arr[j];
				ISA isa1 = new ISA(sx,dx);
				ISA isa2 = new ISA(dx,sx);
				if (!t.getLista_isa().contains(isa1) & !t.getLista_isa().contains(isa2))
					temp = temp.concat("all X ("+sx.getName()+"(X) -> -"+dx.getName()+"(X)).\n");
			}
				
		return temp;
	}
	
	private static String translateInclusionAssertion(TBox t, InclusionAssertion ass) {
		if (ass.isConceptInclusion()) {
			return translateConceptInclusionAssertion(t, ass);
		}
		else if (ass.isRoleInclusion()) {
			return translateRoleInclusionAssertion(ass);
		}
		else if (ass.isValueDomainInclusion()) {
			return translateValueDomainInclusionAssertion(ass);
		}
		else if (ass.isConceptAttributeInclusion()) {
			return translateConceptAttributeInclusionAssertion(ass);
		}
		else if (ass.isRoleAttributeInclusion()) {
			return translateRoleAttributeInclusionAssertion(ass);
			//return "RoleAttributeInclusionAssertion Not Recognized";
       		}
		else return null;
	}
	
	private static String translateConceptAttributeInclusionAssertion(InclusionAssertion ass) {
		AtomicConceptAttribute left = (AtomicConceptAttribute)ass.getLeft();
		Including right = ass.getRight();
		if (right instanceof AtomicConceptAttribute) {
			AtomicConceptAttribute dx = (AtomicConceptAttribute)right;
			return "all X ("+left.getName() +"(X) -> "+dx.getName()+"(X)).";
		}
		else if (right instanceof NegatedConceptAttribute) {
			NegatedConceptAttribute dx = (NegatedConceptAttribute)right;
			return "all X ("+left.getName() +"(X) -> -"+dx.getConceptAttribute().getName()+"(X)).";
		}
		else return "ConceptAttributeInclusionAssertion Not Recognized";
	}
	
	private static String translateRoleAttributeInclusionAssertion(InclusionAssertion ass) {
		AtomicRoleAttribute left = (AtomicRoleAttribute)ass.getLeft();
		Including right = ass.getRight();
		if (right instanceof AtomicRoleAttribute) {
			AtomicRoleAttribute dx = (AtomicRoleAttribute)right;
			return "all X ("+left.getName() +"(X) -> "+dx.getName()+"(X)).";
		}
		else if (right instanceof NegatedRoleAttribute) {
			NegatedRoleAttribute dx = (NegatedRoleAttribute)right;
			return "all X ("+left.getName() +"(X) -> -"+dx.getRoleAttribute().getName()+"(X)).";
		}
		else return "ConceptAttributeInclusionAssertion Not Recognized";
	}


	private static String translateRoleInclusionAssertion(InclusionAssertion ass) {
		Included left = ass.getLeft();
		Including right = ass.getRight();
		if ((left instanceof DirectRoleAttributeDomain)&(right instanceof DirectRole)) {
			DirectRoleAttributeDomain deltaUr = (DirectRoleAttributeDomain)left;
			DirectRole p = (DirectRole)right;
			return "all XYZ ("+deltaUr.getRoleAttribute().getName() +"(X,Y,Z) -> "+p.getRole().getName()+"(X,Y)).";
		}
		else if ((left instanceof DirectRole)&(right instanceof DirectRoleAttributeDomain)) {
			DirectRole p = (DirectRole)left;
			DirectRoleAttributeDomain deltaUr = (DirectRoleAttributeDomain)right;
			return "all XY ("+p.getRole().getName() +"(X,Y) -> (exists Z "+deltaUr.getRoleAttribute().getName()+"(X,Y,Z))).";
		}
		else if ((left instanceof DirectRole)&(right instanceof DirectRole)) {
			DirectRole sx = (DirectRole)left;
			DirectRole dx = (DirectRole)right;
			return "all XY ("+sx.getRole().getName() +"(X,Y) -> "+dx.getRole().getName()+"(X,Y)).";
		}
		else if ((left instanceof DirectRole)&(right instanceof InverseRole)) {
			DirectRole sx = (DirectRole)left;
			InverseRole dx = (InverseRole)right;
			return "all XY ("+sx.getRole().getName() +"(X,Y) -> "+dx.getRole().getName()+"(Y,X)).";
		}
		else if ((left instanceof DirectRole)&(right instanceof NegatedBasicRole)) {
			DirectRole sx = (DirectRole)left;
			NegatedBasicRole dx = (NegatedBasicRole)right;
			BasicRole dxbasic = dx.getBasicRole();
			if (dxbasic instanceof DirectRole) {
				DirectRole p = (DirectRole)dxbasic;
				return "all XY ("+sx.getRole().getName() +"(X,Y) -> -"+p.getRole().getName()+"(X,Y)).";
			}
			else if (dxbasic instanceof InverseRole) {
				InverseRole p = (InverseRole)dxbasic;
				return "all XY ("+sx.getRole().getName() +"(X,Y) -> -"+p.getRole().getName()+"(Y,X)).";
			}
			else return "RoleInclusionAssertion Not Recognized";
		}
		//vedere InverseRole e InverseRoleAttributeDomain
		else return "RoleInclusionAssertion Not Recognized";
	}

	private static String translateFunctionalityAssertion(FunctionalityAssertion ass) {
		Functionable elem = ass.getElem();
		if(elem instanceof AtomicConceptAttribute) {
			AtomicConceptAttribute uc = (AtomicConceptAttribute)elem;
			return "all XYZ ("+ uc.getName() +"(X,Y) & "+ uc.getName() + "(X,Z) -> Y=Z).";
		}
		else if(elem instanceof AtomicRoleAttribute) {
			AtomicRoleAttribute ur = (AtomicRoleAttribute)elem;
			return "all XYZW ("+ ur.getName() +"(X,Y,Z) & "+ ur.getName() + "(X,Y,W) -> Z=W).";
		}
		else if(elem instanceof DirectRole) {
			DirectRole p = (DirectRole)elem;
			return "all XYZ ("+ p.getRole().getName() +"(X,Y) & "+ p.getRole().getName() + "(X,Z) -> Y=Z).";
		}
		else if(elem instanceof InverseRole) {
			InverseRole pmeno = (InverseRole)elem;
			return "all XYZ ("+ pmeno.getRole().getName() +"(Y,X) & "+ pmeno.getRole().getName() + "(Z,X) -> Y=Z).";
		}
		else return "FunctionalityInclusionAssertion Not Recognized";
	}
	
	private static String translateIdentificationAssertion(IdentificationAssertion ass) {
		BasicConcept b = ass.getConcept();
		Collection<AtomicConceptAttribute> ida = ass.getIdlist_attr();
		Collection<BasicRole> idr = ass.getIdlist_role();
		if (b instanceof AtomicConcept) {
			AtomicConcept a = (AtomicConcept)b;
			if (idr == null & ida.size() == 1) {
				AtomicConceptAttribute[] arr = (AtomicConceptAttribute[])ida.toArray();
				return "all XYZ ("+ a.getName() +"(X) & "+a.getName()+"(Y) &"+arr[0].getName()+"(X,Z) & "+arr[0].getName()+"(Y,Z) -> X=Y).";
			}
			else if (ida == null & idr.size() == 1) {
				BasicRole[] arr = (BasicRole[])idr.toArray();
				if (arr[0] instanceof DirectRole) {
					DirectRole p = (DirectRole)arr[0];
					return "all XYZ ("+ a.getName() +"(X) & "+a.getName()+"(Y) &"+p.getRole().getName()+"(X,Z) & "+p.getRole().getName()+"(Y,Z) -> X=Y).";
				}
				else  return "IdentificationAssertion Not Recognized";
			}
			else  return "IdentificationAssertion Not Recognized";
		}
		else return "IdentificationAssertion Not Recognized";
	}
	
	private static String translateConceptInclusionAssertion(TBox t, InclusionAssertion ass) {
		Included left = ass.getLeft();
		Including right = ass.getRight();
		if ((left instanceof ConceptAttributeDomain)&(right instanceof AtomicConcept)) {
			ConceptAttributeDomain deltaUc = (ConceptAttributeDomain)left;
			AtomicConcept a = (AtomicConcept)right;
			return "all XY ("+deltaUc.getConceptAttribute().getName() +"(X,Y) -> "+a.getName()+"(X)).";
		}
		else if ((left instanceof AtomicConcept)&(right instanceof ConceptAttributeDomain)) {
			ConceptAttributeDomain deltaUc = (ConceptAttributeDomain)right;
			AtomicConcept a = (AtomicConcept)left;
			return "all X ("+a.getName() +"(X) -> (exists Y "+deltaUc.getConceptAttribute().getName()+"(X,Y))).";
		}
		else if ((left instanceof AtomicConcept)&(right instanceof AtomicConcept)) {
			AtomicConcept sx = (AtomicConcept)left;
			AtomicConcept dx = (AtomicConcept)right;
			t.addISA(new ISA(sx,dx));
			return "all X ("+sx.getName() +"(X) -> "+dx.getName()+"(X)).";
		}
		else if ((left instanceof AtomicConcept)&(right instanceof NegatedBasicConcept)) {
			AtomicConcept sx = (AtomicConcept)left;
			NegatedBasicConcept dx = (NegatedBasicConcept)right;
			BasicConcept b = dx.getBasicConcept();
			if (b instanceof AtomicConcept) {
				AtomicConcept a = (AtomicConcept)b;
				return "all X ("+sx.getName() +"(X) -> -"+a.getName()+"(X)).";
			}
			else return "ConceptInclusionAssertion Not Recognized";
		}
		else if ((left instanceof ExistentialRole)&(right instanceof AtomicConcept)) {
			ExistentialRole esisteQ = (ExistentialRole)left;
			AtomicConcept a = (AtomicConcept)right;
			BasicRole q = esisteQ.getBasicRole();
			if (q instanceof DirectRole) {
				DirectRole p = (DirectRole)q;
				return "all XY ("+p.getRole().getName() +"(X,Y) -> "+a.getName()+"(X)).";
			}
			else if (q instanceof InverseRole) {
				InverseRole pmeno = (InverseRole)q;
				return "all XY ("+pmeno.getRole().getName() +"(X,Y) -> "+a.getName()+"(Y)).";
			}
			//else esiste delta(ur) ISA A da vedere
			else return "ConceptInclusionAssertion Not Recognized";
		}
		else if ((left instanceof AtomicConcept)&(right instanceof ExistentialRole)) {
			AtomicConcept a = (AtomicConcept)left;
			ExistentialRole esisteQ = (ExistentialRole)right;
			BasicRole q = esisteQ.getBasicRole();
			if (q instanceof DirectRole) {
				DirectRole p = (DirectRole)q;
				return "all XY ("+a.getName() +"(X) -> (exists Y "+p.getRole().getName()+"(X,Y))).";
			}
			else if (q instanceof InverseRole) {
				InverseRole pmeno = (InverseRole)q;
				return "all XY ("+a.getName() +"(X) -> (exists Y "+pmeno.getRole().getName()+"(Y,X))).";
			}
			else return "ConceptInclusionAssertion Not Recognized";
		}
		else return "ConceptInclusionAssertion Not Recognized";
	}
	
	private static String translateValueDomainInclusionAssertion(InclusionAssertion ass) {
		Included left = ass.getLeft();
		Including right = ass.getRight();
		if ((left instanceof ConceptAttributeRange)&(right instanceof AtomicValueDomain)) {
			ConceptAttributeRange roUc = (ConceptAttributeRange)left;
			AtomicValueDomain t = (AtomicValueDomain)right;
			return "all XY ("+roUc.getConceptAttribute().getName() +"(X,Y) -> "+t.getName()+"(Y)).";
		}
		else if ((left instanceof RoleAttributeRange)&(right instanceof AtomicValueDomain)) {
			RoleAttributeRange roUr = (RoleAttributeRange)left;
			AtomicValueDomain t = (AtomicValueDomain)right;
			return "all XYZ ("+roUr.getRoleAttribute().getName() +"(X,Y,Z) -> "+t.getName()+"(Z)).";
		}
		else return "ValueDomainInclusionAssertion Not Recognized";
	}
}
